"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const BN = require("bn.js");
const ABI = require("ethereumjs-abi");
const multihash_1 = require("./multihash");
class Mining {
    constructor(context, feeRecipient, miner, sig) {
        this.context = context;
        this.feeRecipient = feeRecipient;
        this.miner = miner;
        this.sig = sig;
    }
    updateMinerAndInterceptor() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.miner) {
                this.miner = this.feeRecipient;
            }
            else {
                // const [registered, interceptor] = await this.context.minerBrokerRegistry.getBroker(
                //   this.feeRecipient,
                //   this.miner,
                // );
                // // assert(registered, "miner unregistered");
                // if (registered) {
                //   this.interceptor = interceptor;
                // }
            }
        });
    }
    updateHash(rings) {
        let ringHashes = rings[0].hash;
        for (let i = 1; i < rings.length; i++) {
            ringHashes = this.xor(ringHashes, rings[i].hash);
        }
        const args = [
            this.feeRecipient,
            this.miner ? this.miner : "0x0",
            "0x" + ringHashes.toString("hex"),
        ];
        const argTypes = [
            "address",
            "address",
            "bytes32",
        ];
        this.hash = ABI.soliditySHA3(argTypes, args);
    }
    checkMinerSignature(transactionOrigin) {
        const multiHashUtil = new multihash_1.MultiHashUtil();
        if (!this.sig) {
            return (transactionOrigin === this.miner);
        }
        else {
            return multiHashUtil.verifySignature(this.miner, this.hash, this.sig);
        }
    }
    xor(s1, s2) {
        assert(s1.length === s2.length, "can't xor buffers of unequal length");
        const x1 = new BN(s1.toString("hex"), 16);
        const x2 = new BN(s2.toString("hex"), 16);
        const result = x1.xor(x2);
        return new Buffer(result.toString(16, s1.length), "hex");
    }
}
exports.Mining = Mining;
//# sourceMappingURL=mining.js.map